// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/shellcc/platform/shell/programmersguide/shell_adv/scrnsave.asp
#include <windows.h>
#include <commctrl.h>
#include <scrnsave.h>
#include "registry.h"
#include "myres.h"

#define REG_KEY "Software\\HomeMadeApps\\MyScreenSaver"

BITMAPINFOHEADER CreateHeader(int aWidth, int aHeight, WORD aBitPixel)
{
   BITMAPINFOHEADER BitmapInfoHeader;
   BitmapInfoHeader.biSize = sizeof(BITMAPINFOHEADER);
   BitmapInfoHeader.biWidth = aWidth;
   BitmapInfoHeader.biHeight = aHeight;
   BitmapInfoHeader.biPlanes = 1;
   BitmapInfoHeader.biBitCount = aBitPixel;
   BitmapInfoHeader.biCompression = BI_RGB;
   BitmapInfoHeader.biSizeImage = 0;
   BitmapInfoHeader.biXPelsPerMeter = 0;
   BitmapInfoHeader.biYPelsPerMeter = 0;
   BitmapInfoHeader.biClrUsed = 0;
   BitmapInfoHeader.biClrImportant = 0;
   return BitmapInfoHeader;
}

BITMAPINFOHEADER Header;
char *Buffer;
int Width, Height, BytesPixel;
unsigned int Idx;

void Draw(HDC dc)
{
   StretchDIBits(dc,
                 0, Idx, Width, Height - Idx,        // Dest
                 0, Idx, Width, Height - Idx,        // Src
                 Buffer,
                 (LPBITMAPINFO)&Header, DIB_RGB_COLORS, SRCCOPY);
   StretchDIBits(dc,
                 0, 0, Width, Idx,        // Dest
                 0, 0, Width, Idx,        // Src
                 Buffer,
                 (LPBITMAPINFO)&Header, DIB_RGB_COLORS, SRCCOPY);
}

LRESULT WINAPI ScreenSaverProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   switch(message)
   {
   case WM_CREATE:
      {
         HDC dc = GetDC(0);
         Width = GetDeviceCaps(dc, HORZRES);
         Height = GetDeviceCaps(dc, VERTRES);
         HDC MemDc = CreateCompatibleDC(dc);
         HBITMAP BitMap = CreateCompatibleBitmap(dc, Width, Height);
         SelectObject(MemDc, BitMap);
         BitBlt(MemDc, 0, 0, Width, Height, dc, 0, 0, SRCCOPY);
         BITMAP Bm;
         GetObject(BitMap, sizeof(Bm), &Bm);
         Header = CreateHeader(Width, Height, Bm.bmBitsPixel);
         Buffer = new char [Width*Height*Bm.bmBitsPixel/8];
         BytesPixel = Bm.bmBitsPixel/8;
         GetDIBits(MemDc, BitMap, 0, Height, Buffer, (BITMAPINFO *)&Header, DIB_RGB_COLORS);
         ReleaseDC(0, dc);
         ReleaseDC(0, MemDc);
         DeleteObject(BitMap);
         RegistryClass Registry(REG_KEY);
         int Speed = Registry.GetInt("Speed", 10);
         if(Speed < 1)
            Speed = 10;
         SetTimer(hwnd, IDT_TIMER, 1000/Speed, 0);
      }
      break;
   case WM_DESTROY:
      delete Buffer;
      PostQuitMessage(0);
      break;
   case WM_TIMER:
      {
         HDC dc = GetDC(0);
         Idx = (Idx + 1)%Height;
         Draw(dc);
         ReleaseDC(0, dc);
      }
      break;
   case WM_PAINT:
      {
         PAINTSTRUCT ps;
         HDC dc = BeginPaint(hwnd, &ps);
         Draw(dc);
         EndPaint(hwnd, &ps);
      }
      break;
   default:
      return DefScreenSaverProc(hwnd, message, wParam, lParam);
   }
   return 0;
}

BOOL WINAPI ScreenSaverConfigureDialog(HWND aDlg, UINT aMessage, WPARAM wParam, LPARAM)
{
   switch(aMessage)
   {
   case WM_INITDIALOG:
      {
         RegistryClass Registry(REG_KEY);
         int Speed = Registry.GetInt("Speed", 10);
         InitCommonControls();
         SetDlgItemInt(aDlg, IDC_EDIT, Speed, FALSE);
         SendDlgItemMessage(aDlg,
                            IDC_SPIN,
                            UDM_SETRANGE,
                            0,
                            MAKELONG(100, 1));
      }
      return TRUE;
   case WM_COMMAND:
      switch(LOWORD(wParam))
      {
      case IDOK:
         {
            int Speed = GetDlgItemInt(aDlg, IDC_EDIT, 0, FALSE);
            RegistryClass Registry(REG_KEY);
            Registry.WriteInt("Speed", Speed);
            EndDialog(aDlg, TRUE);
         }
         return TRUE;
      case IDCANCEL:
         EndDialog(aDlg, FALSE);
         return TRUE;
      }
      break;
   }
   return FALSE;
}

BOOL WINAPI RegisterDialogClasses(HANDLE)
{
    return TRUE;
}
